package samples.webcenter.portlets.discussions;

import com.jivesoftware.forum.webservices.Forum;
import com.jivesoftware.forum.webservices.ForumCategory;
import com.jivesoftware.forum.webservices.ForumMessage;
import com.jivesoftware.forum.webservices.ForumThread;
import com.jivesoftware.forum.webservices.ResultFilter;
import com.jivesoftware.forum.webservices.User;
import com.jivesoftware.webservices.ContentServicePortType;
import com.jivesoftware.webservices.ForumCategoryServicePortType;
import com.jivesoftware.webservices.ForumMessageServicePortType;
import com.jivesoftware.webservices.ForumServicePortType;
import com.jivesoftware.webservices.ForumThreadServicePortType;
import com.jivesoftware.webservices.PermissionServicePortType;
import com.jivesoftware.webservices.ProfileServicePortType;
import com.jivesoftware.ws.client.ServiceConfig;
import com.jivesoftware.ws.client.ServiceLocator;

import java.io.File;
import java.io.IOException;

import java.io.InputStream;

import java.net.URL;

import java.util.Properties;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletContext;
import javax.portlet.PortletException;
import javax.portlet.PortletMode;
import javax.portlet.PortletPreferences;
import javax.portlet.PortletRequest;
import javax.portlet.PortletResponse;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;


public class DiscussionsPortlet extends GenericPortlet {

	public static String URL = "url";
	public static String CATEGORY_ID = "category_id"; // ID of the category to view
	public static String FORUM_ID = "forum_id"; // ID of the forum to view
	public static String THREAD_ID = "thread_id"; // ID of the thread to view
	public static String MESSAGE_ID = "message_id"; // ID of the message to view
	public static String VIEW_MODE = "view_mode"; // The current view mode (forums, threads, messages)
	public static String PAGE_SIZE = "page_size"; // The number of items to show per page (1-n, <0 == all)
	public static String TEMPLATE = "template"; // Name of template to use
	
	public static int MAX_RESULTS = 30; // The maximun number of items to get (maybe make property/pref?)
	
	public static String[] prefNames = {
		URL,
		CATEGORY_ID,
		FORUM_ID,
		THREAD_ID,
		MESSAGE_ID,
		VIEW_MODE,
		PAGE_SIZE,
		TEMPLATE
	};

	public void setupPrefs(PortletRequest request, PortletResponse response)
		throws PortletException, IOException {

		// Make preferences available to JSPs
		PortletPreferences prefs = request.getPreferences();
		PortletSession session = request.getPortletSession();
		String name = null;
		String value = null;
		String sValue = null;
		
		for (int i = 0; i < prefNames.length; i++) {
			name = prefNames[i];

			value = prefs.getValue(name, null);
			sValue = (String) session.getAttribute(name);
			
			if (request.getUserPrincipal() == null && sValue != null) {
				value = sValue;
			}

			session.setAttribute(name, value);

			request.setAttribute("discussions_" + name, value);
		}
		
		request.setAttribute("discussions_settings", request.getPortletSession().getAttribute("discussions_settings"));
		request.getPortletSession().removeAttribute("discussions_settings");
		
	}
        
        public String cleanUserName(String username){
            System.out.println("Discussions inComingName = " + username);
            String cleanName = username.replace("[1]", "");
            System.out.println("Discussions cleanUserName = " + cleanName);
            return cleanName;
        }

	public ServiceLocator getLocator(PortletRequest request, PortletResponse response)
		throws PortletException, IOException {
	
		String username = "anonymous";
		if (request.getUserPrincipal() != null) {
		     username = cleanUserName(request.getUserPrincipal().getName());
		}
                System.out.println("Discussions Re-Check ** Username = " + username);
		//String password = "";

		String url = (String) request.getPortletSession().getAttribute(URL);
		String discussionsProxyUrl = url != null ? url : null;
		System.out.println("discussionsProxyUrl = " + discussionsProxyUrl);
		ServiceLocator locator = null;
		
		try {
//			ServiceConfig serviceConfig = new ServiceConfig(discussionsProxyUrl, username);
//			locator = new ServiceLocator(serviceConfig);
                        locator = new ServiceLocator(getServiceConfig(discussionsProxyUrl, username));
		} catch (Exception e) {
			request.setAttribute("error_type", "invalid_service");
		}

		try {
			PermissionServicePortType permissionService = locator.getPermissionService();
			long perms = permissionService.getPermissions();
			ForumPerms forumPerms = new ForumPerms(perms);
			request.setAttribute("forum_perms", forumPerms);
		} catch (Exception e) {
			//e.printStackTrace();
			if (e.getMessage() != null) {
				if (e.getMessage().contains("404")) {
					request.setAttribute("error_type", "invalid_service");
				} else if (e.getMessage().toLowerCase().contains("connection refused")) {
					request.setAttribute("error_type", "invalid_url");			
				} else if (e.getMessage().toLowerCase().contains("authentication failed")) {
					request.setAttribute("error_type", "invalid_user");	
				} else {
					request.setAttribute("error_type", "other");
				}
			} else {
				request.setAttribute("error_type", "invalid_url");			
			}
			locator = null;
		}
		
		return locator;
	}
	
	public void setupForums(PortletRequest request, PortletResponse response)
		throws PortletException, IOException {
	
		ServiceLocator locator = this.getLocator(request, response);
	
		if (locator != null) {
			ForumServicePortType forumService = null;

			try {
				forumService = locator.getForumService();
			} catch (Exception e) {
				request.setAttribute("error_type", "invalid_service");
				return;
			}
			
			long categoryId = 1;
			try {
				String c = (String) request.getPortletSession().getAttribute(CATEGORY_ID);
				if (c != null && !"".equals(c)) {
					categoryId = Long.parseLong(c);
				}
			} catch (Exception e) {
				request.setAttribute("error_type", "invalid_category");
				return;
			}
			
			try {
				ResultFilter filter = new ResultFilter();
				filter.setStartIndex(0);
				filter.setNumResults(MAX_RESULTS);
				filter.setSortOrder(1);

				ForumCategoryServicePortType categoryService = locator.getForumCategoryService();
				ForumCategory category = categoryService.getForumCategory(categoryId);
				request.setAttribute("category", category);

				Forum[] forums = forumService.getForumsByCategoryIDWithFilter(categoryId, filter);
				request.setAttribute("forums", forums);
			} catch (Exception e) {
				request.setAttribute("error_type", "invalid_category");
				return;
			}
		}
	}

	public void setupThreads(PortletRequest request, PortletResponse response)
		throws PortletException, IOException {

		ServiceLocator locator = this.getLocator(request, response);
		if (locator != null) {
			ForumServicePortType forumService = null;
			ForumThreadServicePortType forumThreadService = null;
			ForumCategoryServicePortType categoryService = null;
			
			try {
				forumService = locator.getForumService();
				forumThreadService = locator.getForumThreadService();
				categoryService = locator.getForumCategoryService();
			} catch (Exception e) {
				request.setAttribute("error_type", "invalid_service");
				return;
			}
			
			long categoryId = 1;
			try {
				String c = (String) request.getPortletSession().getAttribute(CATEGORY_ID);
				if (c != null && !"".equals(c)) {
					categoryId = Long.parseLong(c);
				}
			} catch (Exception e) {
				request.setAttribute("error_type", "invalid_category");
				return;
			}

			try {
				ForumCategory category = categoryService.getForumCategory(categoryId);
				request.setAttribute("category", category);
			} catch (Exception e) {
				request.setAttribute("error_type", "invalid_category");
				return;
			}

			long forumId = 1;
			try {
				String f = (String) request.getPortletSession().getAttribute(FORUM_ID);
				if (f != null && !"".equals(f)) {
					forumId = Long.parseLong(f);
				}
			} catch (Exception e) {
				request.setAttribute("error_type", "invalid_forum");
				return;
			}

			try {
				Forum forum = forumService.getForum(forumId);
				request.setAttribute("forum", forum);
				ForumThread[] threads = forumThreadService.getThreadsByForumID(forumId);
				request.setAttribute("threads", threads);
			} catch (Exception e) {
				request.setAttribute("error_type", "invalid_forum");
				return;
			}
		} else {
			request.setAttribute("error_type", "invalid_service");
		}
	}

	public void setupMessages(PortletRequest request, PortletResponse response)
		throws PortletException, IOException {
	
		ServiceLocator locator = this.getLocator(request, response);
		if (locator != null) {
			ForumServicePortType forumService = null;
			ForumThreadServicePortType forumThreadService = null;
			ForumCategoryServicePortType categoryService = null;
			ForumMessageServicePortType forumMessageService = locator.getForumMessageService();
			
			try {
				forumService = locator.getForumService();
				forumThreadService = locator.getForumThreadService();
				categoryService = locator.getForumCategoryService();
				forumMessageService = locator.getForumMessageService();
			} catch (Exception e) {
				request.setAttribute("error_type", "invalid_service");
				return;
			}

			long categoryId = 1;
			try {
				String c = (String) request.getPortletSession().getAttribute(CATEGORY_ID);
				if (c != null && !"".equals(c)) {
					categoryId = Long.parseLong(c);
				}
			} catch (Exception e) {
				request.setAttribute("error_type", "invalid_category");
				return;
			}

			try {
				ForumCategory category = categoryService.getForumCategory(categoryId);
				request.setAttribute("category", category);
			} catch (Exception e) {
				request.setAttribute("error_type", "invalid_category");
				return;
			}

			long forumId = 1;
			try {
				String f = (String) request.getPortletSession().getAttribute(FORUM_ID);
				if (f != null && !"".equals(f)) {
					forumId = Long.parseLong(f);
				}
			} catch (Exception e) {
				request.setAttribute("error_type", "invalid_forum");
				return;
			}

			try {
				Forum forum = forumService.getForum(forumId);
				request.setAttribute("forum", forum);
				ForumThread[] threads = forumThreadService.getThreadsByForumID(forumId);
				request.setAttribute("threads", threads);
			} catch (Exception e) {
				request.setAttribute("error_type", "invalid_forum");
				return;
			}

			long threadId = 1;
			try {
				String t = (String) request.getPortletSession().getAttribute(THREAD_ID);
				if (t != null && !"".equals(t)) {
					threadId = Long.parseLong(t);
				}
			} catch (Exception e) {
				request.setAttribute("error_type", "invalid_thread");
				return;
			}

			try {
				ForumThread thread = forumThreadService.getForumThread(threadId);
				request.setAttribute("thread", thread);

				ForumMessage[] messages = forumMessageService.getMessagesByThreadID(threadId);
				String path = request.getContextPath() + "/samples/webcenter/portlets/discussions/";
				for (int i = 0; i < messages.length; i++) {
					String body = messages[i].getBody();
					body = body.replaceAll("images/emoticons", path + "images/emoticons");
					messages[i].setBody(body);
				}
				request.setAttribute("messages", messages);
			} catch (Exception e) {
				request.setAttribute("error_type", "invalid_thread");
				return;
			}

		} else {
			request.setAttribute("error_type", "invalid_service");
		}
	}

	public void doView(RenderRequest request, RenderResponse response)
		throws PortletException, IOException {
		response.setContentType("text/html");

		this.setupPrefs(request, response);

		try {
			String viewMode = (String) request.getPortletSession().getAttribute(VIEW_MODE);
			viewMode = viewMode == null ? "forums" : viewMode;

			if ("forums".equals(viewMode)) {
				this.setupForums(request, response);
			} else if ("threads".equals(viewMode)) {
				this.setupThreads(request, response);
			} else if ("messages".equals(viewMode)) {
				this.setupMessages(request, response);
			}
		} catch (Exception e) {
			//e.printStackTrace();
			request.setAttribute("error_type", "configure");
		}

		PortletContext context = getPortletConfig().getPortletContext();
		String path = "/samples/webcenter/portlets/discussions/view.jsp";
		context.getRequestDispatcher(path).include(request, response);
	}
	
	public void doEdit(RenderRequest request, RenderResponse response)
		throws PortletException, java.io.IOException {

		response.setContentType("text/html");
		this.setupPrefs(request, response);
		PortletContext context = getPortletConfig().getPortletContext();
		String path = "/samples/webcenter/portlets/discussions/edit.jsp";
		context.getRequestDispatcher(path).include(request, response);
	}

	public void doHelp(RenderRequest request, RenderResponse response)
		throws PortletException, java.io.IOException {

		response.setContentType("text/html");
		PortletContext context = getPortletConfig().getPortletContext();
		String path = "/samples/webcenter/portlets/discussions/help.jsp";
		context.getRequestDispatcher(path).include(request, response);
	}
	
	public void processAction(ActionRequest request, ActionResponse response)
		throws PortletException, java.io.IOException {
					
		if (request.getPortletMode().equals(PortletMode.VIEW)) {

			this.parsePrefs(request, response);

			String action = request.getParameter("discussions_action");
			if (action != null) {
				if ("settings".equals(action)) {
					request.getPortletSession().setAttribute("discussions_settings", "true");
				} else if ("update_message".equals(action)) {
					this.updateMessage(request, response);
				} else if ("reply_message".equals(action)) {
					this.replyMessage(request, response);
				} else if ("create_message".equals(action)) {
					this.createMessage(request, response);
				} else if ("create_thread".equals(action)) {
					this.createThread(request, response);
				} else if ("create_forum".equals(action)) {
					this.createForum(request, response);
				} else if ("create_user".equals(action)) {
					this.createUser(request, response);
				}
			}
		} else {
			if (request.getPortletMode().equals(PortletMode.EDIT)) {
				this.parsePrefs(request, response);
				response.setPortletMode(PortletMode.VIEW);
			}
		}
	}
	
	public void updateMessage(PortletRequest request, PortletResponse response)
		throws PortletException, java.io.IOException {
	
		String id = request.getParameter("message_id");
		String content = request.getParameter("message_content");
	
		ServiceLocator locator = this.getLocator(request, response);
		if (locator != null) {
			ForumMessageServicePortType forumMessageService = locator.getForumMessageService();
			
			long messageId = 0;
			ForumMessage message = null;
			try {
				messageId = Long.parseLong(id);
				message = forumMessageService.getForumMessage(messageId);				
				message.setBody(content);
				forumMessageService.updateForumMessage(message);
				request.setAttribute("updated_message_id", message.getID());
			} catch (Exception e) {
				//e.printStackTrace();
				request.setAttribute("error_type", "message_update_failed");
				return;
			}
		}
	}

	public void replyMessage(PortletRequest request, PortletResponse response)
		throws PortletException, java.io.IOException {
	
		String id = request.getParameter("message_id");
		String content = request.getParameter("message_content");
		String title = request.getParameter("message_title");
	
		ServiceLocator locator = this.getLocator(request, response);
		if (locator != null) {
			ProfileServicePortType profileService = locator.getProfileService();
			ContentServicePortType contentService = locator.getContentService();

			try {
				String username = cleanUserName(request.getUserPrincipal().getName());
				long userId = profileService.getUserByUsername(username).getID();
				
				long parentId = Long.parseLong(id);
				ForumMessage message = contentService.createReplyMessage(title, content, parentId, userId);
				request.setAttribute("new_message_id", message.getID());
			} catch (Exception e) {
				//e.printStackTrace();
				request.setAttribute("error_type", "message_reply_failed");
				return;
			}
		}
	}

	public void createMessage(PortletRequest request, PortletResponse response)
		throws PortletException, java.io.IOException {
	
		String id = request.getParameter("thread_id");
		String content = request.getParameter("message_content");
		String title = request.getParameter("message_title");
	
		ServiceLocator locator = this.getLocator(request, response);
		if (locator != null) {
			ProfileServicePortType profileService = locator.getProfileService();
			ContentServicePortType contentService = locator.getContentService();
	
			try {
				String username = cleanUserName(request.getUserPrincipal().getName());
				long userId = profileService.getUserByUsername(username).getID();
				
				long threadId = Long.parseLong(id);
				ForumMessage message = contentService.createMessage(title, content, threadId, userId);
				request.setAttribute("new_message_id", message.getID());
			} catch (Exception e) {
				//e.printStackTrace();
				request.setAttribute("error_type", "message_create_failed");
				return;
			}
		}
	}

	public void createThread(PortletRequest request, PortletResponse response)
		throws PortletException, java.io.IOException {
	
		String id = request.getParameter("forum_id");
		String content = request.getParameter("message_content");
		String title = request.getParameter("message_title");
	
		ServiceLocator locator = this.getLocator(request, response);
		if (locator != null) {
			ProfileServicePortType profileService = locator.getProfileService();
			ContentServicePortType contentService = locator.getContentService();
	
			try {
				String username = cleanUserName(request.getUserPrincipal().getName());
				long userId = profileService.getUserByUsername(username).getID();
				
				long threadId = Long.parseLong(id);
				ForumThread thread = contentService.createThread(title, content, threadId, userId);
				request.setAttribute("new_thread_id", thread.getID());
			} catch (Exception e) {
				//e.printStackTrace();
				request.setAttribute("error_type", "thread_create_failed");
				return;
			}
		}
	}

	public void createForum(PortletRequest request, PortletResponse response)
		throws PortletException, java.io.IOException {
	
		String id = request.getParameter("category_id");
		String title = request.getParameter("forum_title");
		String description = request.getParameter("forum_description");
	
		ServiceLocator locator = this.getLocator(request, response);
		if (locator != null) {
			ForumServicePortType forumService = locator.getForumService();
			
			try {
				long categoryId = Long.parseLong(id);
				Forum forum = forumService.createForumUnderCategory(title, description, categoryId);
				request.setAttribute("new_forum_id", forum.getID());
			} catch (Exception e) {
				//e.printStackTrace();
				request.setAttribute("error_type", "forum_create_failed");
				return;
			}
		}
	}

	public void createUser(PortletRequest request, PortletResponse response)
		throws PortletException, java.io.IOException {

		if (request.getUserPrincipal() == null) {
			return;
		}
		
		// Use the same user name as the current user
		String username = cleanUserName(request.getUserPrincipal().getName());
		String screenName = request.getParameter("screenname");
		boolean screenNameVisible = "on".equals(request.getParameter("screenname_visible"));
		String password = request.getParameter("password");
		String email = request.getParameter("email");
		boolean emailVisible = "on".equals(request.getParameter("email_visible"));
		
		ServiceLocator locator = this.getLocator(request, response);

		if (locator == null) {
			String uname = "anonymous";
			String url = (String) request.getPortletSession().getAttribute(URL);
			String discussionsProxyUrl = url != null ? url : null;
			
			try {
//				ServiceConfig serviceConfig = new ServiceConfig(discussionsProxyUrl, uname);
//				locator = new ServiceLocator(serviceConfig);
			    locator = new ServiceLocator(getServiceConfig(discussionsProxyUrl, username));
			} catch (Exception e) {
				request.setAttribute("error_type", "invalid_service");
			}
		}
		
		if (locator != null) {
			ProfileServicePortType profileService = locator.getProfileService();
			//PermissionServicePortType permissionService = locator.getPermissionService();
			
			try {
				User user = profileService.createUser(username, password, email);

				user.setName(screenName);
				user.setNameVisible(emailVisible);
				user.setEmailVisible(screenNameVisible);
				
				profileService.updateUser(user);
				
			} catch (Exception e) {
				//e.printStackTrace();
				request.setAttribute("error_type", "user_create_failed");
				return;
			}
		}
		
	}

	public void parsePrefs(PortletRequest request, PortletResponse response)
		throws PortletException, java.io.IOException {
	
		PortletPreferences prefs = request.getPreferences();
		String name = null;
		String value = null;
		
		PortletSession session = request.getPortletSession();
		
		for (int i = 0; i < prefNames.length; i++) {
			name = prefNames[i];
			value = request.getParameter("discussions_" + name);
			if (value != null) {
				prefs.setValue(name, value);
				session.setAttribute(name, value);
			}
		}
		
		try {
			prefs.store();
		} catch (Exception e) {
			//e.printStackTrace();
		}
	}
        
    public ServiceConfig getServiceConfig(String url, String userName) {
        if (userName == null) {
            return new ServiceConfig(url);
        } else {
            Properties connectionConfiguration = getConfiguration();
            if (!Boolean.parseBoolean(connectionConfiguration.getProperty("use.ws.security"))) {
                System.out.println("Not using security ...");
                return new ServiceConfig(url, userName);
            }
            System.out.println("Using security ...");
            String cryptoFilePath = null;
            String keyStorePassword = "clientKeyStorePassword";
            String keyStoreType = "jks";
            String alias = "jive";
            String aliasPassword = "jive_password";
            String fileURL = 
                connectionConfiguration.getProperty("keystore.location");
            if (fileURL == null || fileURL.trim().equals("")) {
                URL keyStoreURL = 
                    Thread.currentThread().getContextClassLoader().getResource("jive_client_certs.keystore");
                // this.getClass().getClassLoader().getResource("jive_client_certs.keystore");
                if (keyStoreURL == null) {
                    return new ServiceConfig(url, userName);
                }
                File keyStoreFile = new File(keyStoreURL.getFile());
                cryptoFilePath = keyStoreFile.getAbsolutePath();
            } else {
                cryptoFilePath = fileURL;
            }
            String password = 
                connectionConfiguration.getProperty("keystore.password");
            if (password != null && !password.trim().equals("")) {
                keyStorePassword = password;
            }
            String type = connectionConfiguration.getProperty("keystore.type");
            if (type != null && !type.trim().equals("")) {
                keyStoreType = type;
            }
            String aliasFromCon = 
                connectionConfiguration.getProperty("cert.key.alias");
            if (aliasFromCon != null && !aliasFromCon.trim().equals("")) {
                alias = aliasFromCon;
            }
            String aliasPasswordFromCon = 
                connectionConfiguration.getProperty("cert.key.password");
            if (aliasPasswordFromCon != null && 
                !aliasPasswordFromCon.trim().equals("")) {
                aliasPassword = aliasPasswordFromCon;
            }
            return new ServiceConfig(url, userName, 
                                     "org.apache.ws.security.components.crypto.Merlin", 
                                     cryptoFilePath, keyStoreType, 
                                     keyStorePassword, alias, aliasPassword);
        }
    }

    private static Properties configurationMap = null;

    private Properties getConfiguration() {
        if (configurationMap == null) {
            configurationMap = new Properties();
            try {
                InputStream stream = 
                    Thread.currentThread().getContextClassLoader().getResourceAsStream("jive-security.properties");
                configurationMap.load(stream);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        return configurationMap;
    }
}
