package samples.webcenter.portlets.iframe.wiki;

import java.io.IOException;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletContext;
import javax.portlet.PortletException;
import javax.portlet.PortletMode;
import javax.portlet.PortletPreferences;
import javax.portlet.PortletRequest;
import javax.portlet.PortletResponse;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;


public class WikiPortlet extends GenericPortlet {

    public static String URL = "url";
    public static String SCOPE = "scope";
    public static String PAGE = "page";
    public static String THEME = "theme";
    public static String INLINE = "inline";

    public static String[] prefNames = { URL, SCOPE, PAGE, THEME, INLINE };

    public void _doIt(RenderRequest request, RenderResponse response,
                      String path) throws PortletException, IOException {

        response.setContentType("text/html");

        // Make preferences available to JSPs
        PortletPreferences prefs = request.getPreferences();
        PortletSession session = request.getPortletSession();
        String name = null;
        String value = null;

        for (int i = 0; i < prefNames.length; i++) {
            name = prefNames[i];

            value = prefs.getValue(name, null);

            if (request.getUserPrincipal() == null) {
                value = (String)session.getAttribute(name);
            } else {
                value = prefs.getValue(name, null);
                session.setAttribute(name, value);
            }

            request.setAttribute("wiki_" + name, value);
        }

        // Construct the iFrame src
        String url = (String)session.getAttribute(URL);
        url = url != null ? url : "";
        String scope = (String)session.getAttribute(SCOPE);
        scope = scope != null ? scope : "";
        String page = (String)session.getAttribute(PAGE);
        page = page != null ? page : "";
        String theme = (String)session.getAttribute(THEME);
        theme = theme != null ? theme : "";
        String inline = (String)session.getAttribute(INLINE);
        inline = inline != null ? inline : "";

        if (url.length() > 0) {
            url =
(url.lastIndexOf("/") == url.length() - 1) ? url = url.substring(0,
                                                                 url.length() -
                                                                 1) : url;
            System.out.println("Wiki org URL = **" + url + "**");
            String cleanUrl = url.trim();
            System.out.println("Wiki cleaned URL = **" + cleanUrl + "**");
            //            String wikiSrc = url + "/page/show.jz";
            String wikiSrc = cleanUrl + "/page/show.jz";
            System.out.println("wikiSrc iFrame = " + wikiSrc);

            wikiSrc += "?scope=" + scope;
            wikiSrc += "&page=" + page;
            wikiSrc += "&theme=" + theme;
            wikiSrc += "&inline=" + inline;
            request.setAttribute("wiki_src", wikiSrc);
        }

        request.setAttribute("wiki_action",
                             session.getAttribute("wiki_action"));
        session.removeAttribute("wiki_action");

        PortletContext context = getPortletConfig().getPortletContext();
        context.getRequestDispatcher(path).include(request, response);
    }

    public void doView(RenderRequest request,
                       RenderResponse response) throws PortletException,
                                                       IOException {

        this._doIt(request, response,
                   "/samples/webcenter/portlets/iframe/wiki/view.jsp");
    }

    public void doEdit(RenderRequest request,
                       RenderResponse response) throws PortletException,
                                                       java.io.IOException {

        this._doIt(request, response,
                   "/samples/webcenter/portlets/iframe/wiki/edit.jsp");
    }


    public void doHelp(RenderRequest request,
                       RenderResponse response) throws PortletException,
                                                       java.io.IOException {

        this._doIt(request, response,
                   "/samples/webcenter/portlets/iframe/wiki/help.jsp");
    }

    public void processAction(ActionRequest request,
                              ActionResponse response) throws PortletException,
                                                              java.io.IOException {

        if (request.getPortletMode().equals(PortletMode.VIEW)) {
            this.parsePrefs(request, response);
            request.getPortletSession().setAttribute("wiki_action",
                                                     request.getParameter("wiki_action"));
        } else {
            if (request.getPortletMode().equals(PortletMode.EDIT)) {
                this.parsePrefs(request, response);
                response.setPortletMode(PortletMode.VIEW);
            }
        }
    }

    public void parsePrefs(PortletRequest request,
                           PortletResponse response) throws PortletException,
                                                            java.io.IOException {

        PortletPreferences prefs = request.getPreferences();
        String name = null;
        String value = null;

        PortletSession session = request.getPortletSession();

        for (int i = 0; i < prefNames.length; i++) {
            name = prefNames[i];
            value = request.getParameter("wiki_" + name);
            if (value != null) {
                prefs.setValue(name, value);
                session.setAttribute(name, value);
            }
        }

        try {
            prefs.store();
        } catch (Exception e) {
            //e.printStackTrace();
        }
    }
}
